@@ -2,6 +2,7 @@
 #   Cython - Command Line Parsing
 #
 
+import os
 import sys
 import Options
 
@@ -23,19 +24,20 @@ Options:
   -v, --verbose                  Be verbose, print file names on multiple compilation
   -p, --embed-positions          If specified, the positions in Cython files of each
                                  function definition is embedded in its docstring.
-  -z, --pre-import <module>      If specified, assume undeclared names in this 
-                                 module. Emulates the behavior of putting 
-                                 "from <module> import *" at the top of the file. 
-  --cleanup <level>              Release interned objects on python exit, for memory debugging. 
-                                 Level indicates aggressiveness, default 0 releases nothing. 
-  -w, --working <directory>      Sets the working directory for Cython (the directory modules 
+  --cleanup <level>              Release interned objects on python exit, for memory debugging.
+                                 Level indicates aggressiveness, default 0 releases nothing.
+  -w, --working <directory>      Sets the working directory for Cython (the directory modules
                                  are searched from)
+  --gdb                          Output debug information for cygdb
 
   -D, --no-docstrings            Remove docstrings.
   -a, --annotate                 Produce a colorized HTML version of the source.
   --line-directives              Produce #line directives pointing to the .pyx source
   --cplus                        Output a c++ rather than c file.
   --embed                        Embed the Python interpreter in a main() method.
+  -2                             Compile based on Python-2 syntax and code semantics.
+  -3                             Compile based on Python-3 syntax and code semantics.
+  --fast-fail                    Abort the compilation on the first error
   -X, --directive <name>=<value>[,<name=value,...] Overrides a compiler directive
 """
 
@@ -63,7 +65,7 @@ def parse_command_line(args):
             return args.pop(0)
         else:
             bad_usage()
-    
+
     def get_param(option):
         tail = option[2:]
         if tail:
@@ -80,11 +82,6 @@ def parse_command_line(args):
                 options.show_version = 1
             elif option in ("-l", "--create-listing"):
                 options.use_listing_file = 1
-            elif option in ("-C", "--compile"):
-                options.c_only = 0
-            elif option in ("--link"):
-                options.c_only = 0
-                options.obj_only = 0
             elif option in ("-+", "--cplus"):
                 options.cplus = 1
             elif option == "--embed":
@@ -119,15 +116,36 @@ def parse_command_line(args):
                 Options.convert_range = True
             elif option == "--line-directives":
                 options.emit_linenums = True
+            elif option == "--gdb":
+                options.gdb_debug = True
+                options.output_dir = os.curdir
+            elif option == '-2':
+                options.language_level = 2
+            elif option == '-3':
+                options.language_level = 3
+            elif option == "--fast-fail":
+                Options.fast_fail = True
+            elif option == "--disable-function-redefinition":
+                Options.disable_function_redefinition = True
             elif option in ("-X", "--directive"):
                 try:
-                    options.compiler_directives = Options.parse_directive_list(pop_arg())
+                    options.compiler_directives = Options.parse_directive_list(
+                        pop_arg(), relaxed_bool=True,
+                        current_settings=options.compiler_directives)
                 except ValueError, e:
-                    sys.stderr.write("Error in compiler directive: %s\n" % e.message)
+                    sys.stderr.write("Error in compiler directive: %s\n" % e.args[0])
                     sys.exit(1)
+            elif option.startswith('--debug'):
+                option = option[2:].replace('-', '_')
+                import DebugFlags
+                if option in dir(DebugFlags):
+                    setattr(DebugFlags, option, True)
+                else:
+                    sys.stderr.write("Unknown debug flag: %s\n" % option)
+                    bad_usage()
             else:
                 sys.stderr.write("Unknown compiler flag: %s\n" % option)
-                bad_usage()
+                sys.exit(1)
         else:
             arg = pop_arg()
             if arg.endswith(".pyx"):
@@ -135,14 +153,9 @@ def parse_command_line(args):
             elif arg.endswith(".py"):
                 # maybe do some other stuff, but this should work for now
                 sources.append(arg)
-            elif arg.endswith(".o"):
-                options.objects.append(arg)
             else:
                 sys.stderr.write(
                     "cython: %s: Unknown filename suffix\n" % arg)
-    if options.objects and len(sources) > 1:
-        sys.stderr.write(
-            "cython: Only one source file allowed together with .o files\n")
     if options.use_listing_file and len(sources) > 1:
         sys.stderr.write(
             "cython: Only one source file allowed when using -o\n")
