@@ -1,7 +1,13 @@
+# cython: language_level = 3
 #
 #   Pyrex - Code output module
 #
 
+import cython
+cython.declare(re=object, Naming=object, Options=object, StringEncoding=object,
+               Utils=object, SourceDescriptor=object, StringIOTree=object,
+               DebugFlags=object, none_or_sub=object)
+
 import re
 import Naming
 import Options
@@ -9,13 +15,13 @@ import StringEncoding
 from Cython import Utils
 from Scanning import SourceDescriptor
 from Cython.StringIOTree import StringIOTree
-try:
-    set
-except NameError:
-    from sets import Set as set
 import DebugFlags
 
 from Cython.Utils import none_or_sub
+try:
+    basestring
+except NameError:
+    basestring = str
 
 class UtilityCode(object):
     # Stores utility code to add during code generation.
@@ -23,7 +29,7 @@ class UtilityCode(object):
     # See GlobalState.put_utility_code.
     #
     # hashes/equals by instance
-    
+
     def __init__(self, proto=None, impl=None, init=None, cleanup=None, requires=None,
                  proto_block='utility_code_proto'):
         # proto_block: Which code block to dump prototype in. See GlobalState.
@@ -40,7 +46,7 @@ class UtilityCode(object):
         # Dicts aren't hashable...
         if pyrex_type is not None:
             data['type'] = pyrex_type.declaration_code('')
-            data['type_name'] = pyrex_type.specalization_name()
+            data['type_name'] = pyrex_type.specialization_name()
         key = data.items(); key.sort(); key = tuple(key)
         try:
             return self._cache[key]
@@ -78,8 +84,8 @@ class UtilityCode(object):
                 writer.put(self.cleanup)
             else:
                 self.cleanup(writer, output.module_pos)
-            
-        
+
+
 
 class FunctionState(object):
     # return_label     string          function return point label
@@ -92,13 +98,13 @@ class FunctionState(object):
     # exc_vars         (string * 3)    exception variables for reraise, or None
 
     # Not used for now, perhaps later
-    def __init__(self, owner, names_taken=set()):
+    def __init__(self, owner, names_taken=cython.set()):
         self.names_taken = names_taken
         self.owner = owner
-        
+
         self.error_label = None
         self.label_counter = 0
-        self.labels_used = {}
+        self.labels_used = cython.set()
         self.return_label = self.new_label()
         self.new_error_label()
         self.continue_label = None
@@ -121,28 +127,28 @@ class FunctionState(object):
         if name is not None:
             label += '_' + name
         return label
-    
+
     def new_error_label(self):
         old_err_lbl = self.error_label
         self.error_label = self.new_label('error')
         return old_err_lbl
-    
+
     def get_loop_labels(self):
         return (
             self.continue_label,
             self.break_label)
-    
+
     def set_loop_labels(self, labels):
         (self.continue_label,
          self.break_label) = labels
-    
+
     def new_loop_labels(self):
         old_labels = self.get_loop_labels()
         self.set_loop_labels(
-            (self.new_label("continue"), 
+            (self.new_label("continue"),
              self.new_label("break")))
         return old_labels
-    
+
     def get_all_labels(self):
         return (
             self.continue_label,
@@ -166,10 +172,10 @@ class FunctionState(object):
                 new_labels.append(old_label)
         self.set_all_labels(new_labels)
         return old_labels
-    
+
     def use_label(self, lbl):
-        self.labels_used[lbl] = 1
-        
+        self.labels_used.add(lbl)
+
     def label_used(self, lbl):
         return lbl in self.labels_used
 
@@ -260,7 +266,7 @@ class FunctionState(object):
         error case.
         """
         return [(cname, type)
-                for (type, manage_ref), freelist in self.temps_free.iteritems()
+                for (type, manage_ref), freelist in self.temps_free.items()
                 if manage_ref
                 for cname in freelist]
 
@@ -287,9 +293,11 @@ class PyObjectConst(object):
         self.cname = cname
         self.type = type
 
+cython.declare(possible_unicode_identifier=object, possible_bytes_identifier=object,
+               nice_identifier=object, find_alphanums=object)
 possible_unicode_identifier = re.compile(ur"(?![0-9])\w+$", re.U).match
 possible_bytes_identifier = re.compile(r"(?![0-9])\w+$".encode('ASCII')).match
-nice_identifier = re.compile('^[a-zA-Z0-9_]+$').match
+nice_identifier = re.compile(r'\A[a-zA-Z0-9_]+\Z').match
 find_alphanums = re.compile('([a-zA-Z0-9]+)').findall
 
 class StringConst(object):
@@ -394,7 +402,7 @@ class GlobalState(object):
 
     # parts            {string:CCodeWriter}
 
-    
+
     # interned_strings
     # consts
     # interned_nums
@@ -407,6 +415,7 @@ class GlobalState(object):
 
     code_layout = [
         'h_code',
+        'filename_table',
         'utility_code_proto_before_types',
         'numeric_typedefs',          # Let these detailed individual parts stay!,
         'complex_type_declarations', # as the proper solution is to make a full DAG...
@@ -420,22 +429,22 @@ class GlobalState(object):
         'all_the_rest',
         'pystring_table',
         'cached_builtins',
+        'cached_constants',
         'init_globals',
         'init_module',
         'cleanup_globals',
         'cleanup_module',
         'main_method',
-        'filename_table',
         'utility_code_def',
         'end'
     ]
-    
+
 
     def __init__(self, writer, emit_linenums=False):
         self.filename_table = {}
         self.filename_list = []
         self.input_file_contents = {}
-        self.utility_codes = set()
+        self.utility_codes = cython.set()
         self.declared_cnames = {}
         self.in_utility_code_generation = False
         self.emit_linenums = emit_linenums
@@ -462,7 +471,12 @@ class GlobalState(object):
             w.enter_cfunc_scope()
             w.putln("static int __Pyx_InitCachedBuiltins(void) {")
 
-        
+        w = self.parts['cached_constants']
+        w.enter_cfunc_scope()
+        w.putln("")
+        w.putln("static int __Pyx_InitCachedConstants(void) {")
+        w.put_setup_refcount_context("__Pyx_InitCachedConstants")
+
         w = self.parts['init_globals']
         w.enter_cfunc_scope()
         w.putln("")
@@ -484,11 +498,6 @@ class GlobalState(object):
             code.write('\n#line 1 "cython_utility"\n')
         code.putln("")
         code.putln("/* Runtime support code */")
-        code.putln("")
-        code.putln("static void %s(void) {" % Naming.fileinit_cname)
-        code.putln("%s = %s;" % 
-            (Naming.filetable_cname, Naming.filenames_cname))
-        code.putln("}")
 
     def finalize_main_c_code(self):
         self.close_global_decls()
@@ -514,15 +523,27 @@ class GlobalState(object):
         if Options.cache_builtins:
             w = self.parts['cached_builtins']
             w.putln("return 0;")
-            w.put_label(w.error_label)
-            w.putln("return -1;")
+            if w.label_used(w.error_label):
+                w.put_label(w.error_label)
+                w.putln("return -1;")
             w.putln("}")
             w.exit_cfunc_scope()
 
+        w = self.parts['cached_constants']
+        w.put_finish_refcount_context()
+        w.putln("return 0;")
+        if w.label_used(w.error_label):
+            w.put_label(w.error_label)
+            w.put_finish_refcount_context()
+            w.putln("return -1;")
+        w.putln("}")
+        w.exit_cfunc_scope()
+
         w = self.parts['init_globals']
         w.putln("return 0;")
-        w.put_label(w.error_label)
-        w.putln("return -1;")
+        if w.label_used(w.error_label):
+            w.put_label(w.error_label)
+            w.putln("return -1;")
         w.putln("}")
         w.exit_cfunc_scope()
 
@@ -535,23 +556,31 @@ class GlobalState(object):
             w = self.parts['cleanup_module']
             w.putln("}")
             w.exit_cfunc_scope()
-         
+
     def put_pyobject_decl(self, entry):
         self['global_var'].putln("static PyObject *%s;" % entry.cname)
 
     # constant handling at code generation time
 
+    def get_cached_constants_writer(self):
+        return self.parts['cached_constants']
+
     def get_int_const(self, str_value, longness=False):
-        longness = bool(longness or Utils.long_literal(str_value))
+        longness = bool(longness)
         try:
             c = self.int_const_index[(str_value, longness)]
         except KeyError:
             c = self.new_int_const(str_value, longness)
         return c
 
-    def get_py_const(self, type):
+    def get_py_const(self, type, prefix='', cleanup_level=None):
         # create a new Python object constant
-        return self.new_py_const(type)
+        const = self.new_py_const(type, prefix)
+        if cleanup_level is not None \
+               and cleanup_level <= Options.generate_cleanup_code:
+            cleanup_writer = self.parts['cleanup_globals']
+            cleanup_writer.put_xdecref_clear(const.cname, type, nanny=False)
+        return const
 
     def get_string_const(self, text):
         # return a C string constant, creating a new one if necessary
@@ -586,8 +615,8 @@ class GlobalState(object):
         self.int_const_index[(value, longness)] = c
         return c
 
-    def new_py_const(self, type):
-        cname = self.new_const_cname()
+    def new_py_const(self, type, prefix=''):
+        cname = self.new_const_cname(prefix)
         c = PyObjectConst(cname, type)
         self.py_constants.append(c)
         return c
@@ -619,15 +648,32 @@ class GlobalState(object):
     def add_cached_builtin_decl(self, entry):
         if Options.cache_builtins:
             if self.should_declare(entry.cname, entry):
-                interned_cname = self.get_interned_identifier(entry.name).cname
                 self.put_pyobject_decl(entry)
                 w = self.parts['cached_builtins']
-                w.putln('%s = __Pyx_GetName(%s, %s); if (!%s) %s' % (
-                    entry.cname,
-                    Naming.builtins_cname,
-                    interned_cname,
-                    entry.cname,
-                    w.error_goto(entry.pos)))
+                if entry.name == 'xrange':
+                    # replaced by range() in Py3
+                    w.putln('#if PY_MAJOR_VERSION >= 3')
+                    self.put_cached_builtin_init(
+                        entry.pos, StringEncoding.EncodedString('range'),
+                        entry.cname)
+                    w.putln('#else')
+                self.put_cached_builtin_init(
+                    entry.pos, StringEncoding.EncodedString(entry.name),
+                    entry.cname)
+                if entry.name == 'xrange':
+                    w.putln('#endif')
+
+    def put_cached_builtin_init(self, pos, name, cname):
+        w = self.parts['cached_builtins']
+        interned_cname = self.get_interned_identifier(name).cname
+        from ExprNodes import get_name_interned_utility_code
+        self.use_utility_code(get_name_interned_utility_code)
+        w.putln('%s = __Pyx_GetName(%s, %s); if (!%s) %s' % (
+            cname,
+            Naming.builtins_cname,
+            interned_cname,
+            cname,
+            w.error_goto(pos)))
 
     def generate_const_declarations(self):
         self.generate_string_constants()
@@ -645,16 +691,16 @@ class GlobalState(object):
 
     def generate_string_constants(self):
         c_consts = [ (len(c.cname), c.cname, c)
-                     for c in self.string_const_index.itervalues() ]
+                     for c in self.string_const_index.values() ]
         c_consts.sort()
         py_strings = []
 
         decls_writer = self.parts['decls']
         for _, cname, c in c_consts:
             decls_writer.putln('static char %s[] = "%s";' % (
-                cname, c.escaped_value))
+                cname, StringEncoding.split_string_literal(c.escaped_value)))
             if c.py_strings is not None:
-                for py_string in c.py_strings.itervalues():
+                for py_string in c.py_strings.values():
                     py_strings.append((c.cname, len(py_string.cname), py_string))
 
         if py_strings:
@@ -697,7 +743,7 @@ class GlobalState(object):
 
     def generate_int_constants(self):
         consts = [ (len(c.value), c.value, c.is_long, c)
-                   for c in self.int_const_index.itervalues() ]
+                   for c in self.int_const_index.values() ]
         consts.sort()
         decls_writer = self.parts['decls']
         for _, value, longness, c in consts:
@@ -705,6 +751,8 @@ class GlobalState(object):
             decls_writer.putln("static PyObject *%s;" % cname)
             if longness:
                 function = '%s = PyLong_FromString((char *)"%s", 0, 0); %s;'
+            elif Utils.long_literal(value):
+                function = '%s = PyInt_FromString((char *)"%s", 0, 0); %s;'
             else:
                 function = "%s = PyInt_FromLong(%s); %s;"
             init_globals = self.parts['init_globals']
@@ -716,7 +764,7 @@ class GlobalState(object):
     # The functions below are there in a transition phase only
     # and will be deprecated. They are called from Nodes.BlockNode.
     # The copy&paste duplication is intentional in order to be able
-    # to see quickly how BlockNode worked, until this is replaced.    
+    # to see quickly how BlockNode worked, until this is replaced.
 
     def should_declare(self, cname, entry):
         if cname in self.declared_cnames:
@@ -745,21 +793,27 @@ class GlobalState(object):
         try:
             return self.input_file_contents[source_desc]
         except KeyError:
+            pass
+        source_file = source_desc.get_lines(encoding='ASCII',
+                                            error_handling='ignore')
+        try:
             F = [u' * ' + line.rstrip().replace(
                     u'*/', u'*[inserted by cython to avoid comment closer]/'
                     ).replace(
                     u'/*', u'/[inserted by cython to avoid comment start]*'
                     )
-                 for line in source_desc.get_lines(encoding='ASCII',
-                                                   error_handling='ignore')]
-            if len(F) == 0: F.append(u'')
-            self.input_file_contents[source_desc] = F
-            return F
+                 for line in source_file]
+        finally:
+            if hasattr(source_file, 'close'):
+                source_file.close()
+        if not F: F.append(u'')
+        self.input_file_contents[source_desc] = F
+        return F
 
     #
     # Utility code state
     #
-    
+
     def use_utility_code(self, utility_code):
         """
         Adds code to the C file. utility_code should
@@ -805,10 +859,10 @@ class CCodeWriter(object):
     - filename_table, filename_list, input_file_contents: All codewriters
       coming from the same root share the same instances simultaneously.
     """
-    
+
     # f                file            output file
     # buffer           StringIOTree
-    
+
     # level            int             indentation level
     # bol              bool            beginning of line?
     # marker           string          comment to emit before next line
@@ -822,7 +876,7 @@ class CCodeWriter(object):
     #                                  about the current class one is in
 
     globalstate = None
-    
+
     def __init__(self, create_from=None, buffer=None, copy_formatting=False, emit_linenums=None):
         if buffer is None: buffer = StringIOTree()
         self.buffer = buffer
@@ -830,7 +884,7 @@ class CCodeWriter(object):
         self.last_marker_line = 0
         self.source_desc = ""
         self.pyclass_stack = []
-        
+
         self.funcstate = None
         self.level = 0
         self.call_level = 0
@@ -862,6 +916,14 @@ class CCodeWriter(object):
         return self.buffer.getvalue()
 
     def write(self, s):
+        # also put invalid markers (lineno 0), to indicate that those lines
+        # have no Cython source code correspondence
+        if self.marker is None:
+            cython_lineno = self.last_marker_line
+        else:
+            cython_lineno = self.marker[0]
+
+        self.buffer.markers.extend([cython_lineno] * s.count('\n'))
         self.buffer.write(s)
 
     def insertion_point(self):
@@ -909,7 +971,7 @@ class CCodeWriter(object):
 
     def enter_cfunc_scope(self):
         self.funcstate = FunctionState(self)
-    
+
     def exit_cfunc_scope(self):
         self.funcstate = None
 
@@ -918,6 +980,9 @@ class CCodeWriter(object):
     def get_py_num(self, str_value, longness):
         return self.globalstate.get_int_const(str_value, longness).cname
 
+    def get_py_const(self, type, prefix='', cleanup_level=None):
+        return self.globalstate.get_py_const(type, prefix, cleanup_level).cname
+
     def get_string_const(self, text):
         return self.globalstate.get_string_const(text).cname
 
@@ -933,6 +998,9 @@ class CCodeWriter(object):
     def intern_identifier(self, text):
         return self.get_py_string_const(text, identifier=True)
 
+    def get_cached_constants_writer(self):
+        return self.globalstate.get_cached_constants_writer()
+
     # code generation
 
     def putln(self, code = "", safe=False):
@@ -940,6 +1008,7 @@ class CCodeWriter(object):
             self.emit_marker()
         if self.emit_linenums and self.last_marker_line != 0:
             self.write('\n#line %s "%s"\n' % (self.last_marker_line, self.source_desc))
+
         if code:
             if safe:
                 self.put_safe(code)
@@ -947,7 +1016,7 @@ class CCodeWriter(object):
                 self.put(code)
         self.write("\n");
         self.bol = 1
-    
+
     def emit_marker(self):
         self.write("\n");
         self.indent()
@@ -985,18 +1054,18 @@ class CCodeWriter(object):
 
     def increase_indent(self):
         self.level = self.level + 1
-    
+
     def decrease_indent(self):
         self.level = self.level - 1
-    
+
     def begin_block(self):
         self.putln("{")
         self.increase_indent()
-    
+
     def end_block(self):
         self.decrease_indent()
         self.putln("}")
-    
+
     def indent(self):
         self.write("  " * self.level)
 
@@ -1020,21 +1089,21 @@ class CCodeWriter(object):
         self.marker = (line, marker)
         if self.emit_linenums:
             self.source_desc = source_desc.get_escaped_description()
-        
+
     def put_label(self, lbl):
         if lbl in self.funcstate.labels_used:
             self.putln("%s:;" % lbl)
-    
+
     def put_goto(self, lbl):
         self.funcstate.use_label(lbl)
         self.putln("goto %s;" % lbl)
-    
+
     def put_var_declarations(self, entries, static = 0, dll_linkage = None,
             definition = True):
         for entry in entries:
             if not entry.in_cinclude:
                 self.put_var_declaration(entry, static, dll_linkage, definition)
-    
+
     def put_var_declaration(self, entry, static = 0, dll_linkage = None,
             definition = True):
         #print "Code.put_var_declaration:", entry.name, "definition =", definition ###
@@ -1077,7 +1146,7 @@ class CCodeWriter(object):
     def put_h_guard(self, guard):
         self.putln("#ifndef %s" % guard)
         self.putln("#define %s" % guard)
-    
+
     def unlikely(self, cond):
         if Options.gcc_branch_hints:
             return 'unlikely(%s)' % cond
@@ -1093,17 +1162,17 @@ class CCodeWriter(object):
             return "(PyObject *)" + entry.cname
         else:
             return entry.cname
-    
+
     def as_pyobject(self, cname, type):
         from PyrexTypes import py_object_type, typecast
         return typecast(py_object_type, type, cname)
-    
+
     def put_gotref(self, cname):
         self.putln("__Pyx_GOTREF(%s);" % cname)
-    
+
     def put_giveref(self, cname):
         self.putln("__Pyx_GIVEREF(%s);" % cname)
-    
+
     def put_xgiveref(self, cname):
         self.putln("__Pyx_XGIVEREF(%s);" % cname)
 
@@ -1115,7 +1184,7 @@ class CCodeWriter(object):
             self.putln("__Pyx_INCREF(%s);" % self.as_pyobject(cname, type))
         else:
             self.putln("Py_INCREF(%s);" % self.as_pyobject(cname, type))
-    
+
     def put_decref(self, cname, type, nanny=True):
         if nanny:
             self.putln("__Pyx_DECREF(%s);" % self.as_pyobject(cname, type))
@@ -1125,7 +1194,7 @@ class CCodeWriter(object):
     def put_var_gotref(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_GOTREF(%s);" % self.entry_as_pyobject(entry))
-        
+
     def put_var_giveref(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_GIVEREF(%s);" % self.entry_as_pyobject(entry))
@@ -1141,7 +1210,7 @@ class CCodeWriter(object):
     def put_var_incref(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_INCREF(%s);" % self.entry_as_pyobject(entry))
-    
+
     def put_decref_clear(self, cname, type, nanny=True):
         from PyrexTypes import py_object_type, typecast
         if nanny:
@@ -1150,13 +1219,13 @@ class CCodeWriter(object):
         else:
             self.putln("Py_DECREF(%s); %s = 0;" % (
                 typecast(py_object_type, type, cname), cname))
-    
+
     def put_xdecref(self, cname, type, nanny=True):
         if nanny:
             self.putln("__Pyx_XDECREF(%s);" % self.as_pyobject(cname, type))
         else:
             self.putln("Py_XDECREF(%s);" % self.as_pyobject(cname, type))
-    
+
     def put_xdecref_clear(self, cname, type, nanny=True):
         if nanny:
             self.putln("__Pyx_XDECREF(%s); %s = 0;" % (
@@ -1167,25 +1236,25 @@ class CCodeWriter(object):
 
     def put_var_decref(self, entry):
         if entry.type.is_pyobject:
-            if entry.init_to_none is False:
+            if entry.init_to_none is False:  # FIXME: 0 and False are treated differently???
                 self.putln("__Pyx_XDECREF(%s);" % self.entry_as_pyobject(entry))
             else:
                 self.putln("__Pyx_DECREF(%s);" % self.entry_as_pyobject(entry))
-    
+
     def put_var_decref_clear(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_DECREF(%s); %s = 0;" % (
                 self.entry_as_pyobject(entry), entry.cname))
-    
+
     def put_var_xdecref(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_XDECREF(%s);" % self.entry_as_pyobject(entry))
-    
+
     def put_var_xdecref_clear(self, entry):
         if entry.type.is_pyobject:
             self.putln("__Pyx_XDECREF(%s); %s = 0;" % (
                 self.entry_as_pyobject(entry), entry.cname))
-    
+
     def put_var_decrefs(self, entries, used_only = 0):
         for entry in entries:
             if not used_only or entry.used:
@@ -1193,15 +1262,15 @@ class CCodeWriter(object):
                     self.put_var_xdecref(entry)
                 else:
                     self.put_var_decref(entry)
-    
+
     def put_var_xdecrefs(self, entries):
         for entry in entries:
             self.put_var_xdecref(entry)
-    
+
     def put_var_xdecrefs_clear(self, entries):
         for entry in entries:
             self.put_var_xdecref_clear(entry)
-    
+
     def put_init_to_py_none(self, cname, type, nanny=True):
         from PyrexTypes import py_object_type, typecast
         py_none = typecast(type, py_object_type, "Py_None")
@@ -1209,14 +1278,23 @@ class CCodeWriter(object):
             self.putln("%s = %s; __Pyx_INCREF(Py_None);" % (cname, py_none))
         else:
             self.putln("%s = %s; Py_INCREF(Py_None);" % (cname, py_none))
-    
+
     def put_init_var_to_py_none(self, entry, template = "%s", nanny=True):
         code = template % entry.cname
         #if entry.type.is_extension_type:
         #    code = "((PyObject*)%s)" % code
         self.put_init_to_py_none(code, entry.type, nanny)
 
-    def put_pymethoddef(self, entry, term):
+    def put_pymethoddef(self, entry, term, allow_skip=True):
+        if entry.is_special or entry.name == '__getattribute__':
+            if entry.name not in ['__cinit__', '__dealloc__', '__richcmp__', '__next__', '__getreadbuffer__', '__getwritebuffer__', '__getsegcount__', '__getcharbuffer__', '__getbuffer__', '__releasebuffer__']:
+                if entry.name == '__getattr__' and not self.globalstate.directives['fast_getattr']:
+                    pass
+                # Python's typeobject.c will automatically fill in our slot
+                # in add_operators() (called by PyType_Ready) with a value
+                # that's better than ours.
+                elif allow_skip:
+                    return
         from TypeSlots import method_coexist
         if entry.doc:
             doc_code = entry.doc_cname
@@ -1228,7 +1306,7 @@ class CCodeWriter(object):
                 method_flags += [method_coexist]
             self.putln(
                 '{__Pyx_NAMESTR("%s"), (PyCFunction)%s, %s, __Pyx_DOCSTR(%s)}%s' % (
-                    entry.name, 
+                    entry.name,
                     entry.func_cname,
                     "|".join(method_flags),
                     doc_code,
@@ -1252,7 +1330,7 @@ class CCodeWriter(object):
             Naming.lineno_cname,
             pos[1],
             cinfo)
-        
+
     def error_goto(self, pos):
         lbl = self.funcstate.error_label
         self.funcstate.use_label(lbl)
@@ -1262,16 +1340,16 @@ class CCodeWriter(object):
 
     def error_goto_if(self, cond, pos):
         return "if (%s) %s" % (self.unlikely(cond), self.error_goto(pos))
-            
+
     def error_goto_if_null(self, cname, pos):
         return self.error_goto_if("!%s" % cname, pos)
-    
+
     def error_goto_if_neg(self, cname, pos):
         return self.error_goto_if("%s < 0" % cname, pos)
-    
+
     def error_goto_if_PyErr(self, pos):
         return self.error_goto_if("PyErr_Occurred()", pos)
-    
+
     def lookup_filename(self, filename):
         return self.globalstate.lookup_filename(filename)
 
@@ -1281,12 +1359,15 @@ class CCodeWriter(object):
     def put_finish_refcount_context(self):
         self.putln("__Pyx_RefNannyFinishContext();")
 
+    def put_trace_declarations(self):
+        self.putln('__Pyx_TraceDeclarations');
+
     def put_trace_call(self, name, pos):
         self.putln('__Pyx_TraceCall("%s", %s[%s], %s);' % (name, Naming.filetable_cname, self.lookup_filename(pos[0]), pos[1]));
-    
+
     def put_trace_exception(self):
         self.putln("__Pyx_TraceException();")
-    
+
     def put_trace_return(self, retvalue_cname):
         self.putln("__Pyx_TraceReturn(%s);" % retvalue_cname)
 
@@ -1298,13 +1379,13 @@ class PyrexCodeWriter(object):
     def __init__(self, outfile_name):
         self.f = Utils.open_new_file(outfile_name)
         self.level = 0
-    
+
     def putln(self, code):
         self.f.write("%s%s\n" % (" " * self.level, code))
-    
+
     def indent(self):
         self.level += 1
-    
+
     def dedent(self):
         self.level -= 1
 
