@@ -2,7 +2,7 @@
 #   Cython Top Level
 #
 
-import os, sys, re, codecs
+import os, sys, re
 if sys.version_info[:2] < (2, 3):
     sys.stderr.write("Sorry, Cython requires Python 2.3 or later\n")
     sys.exit(1)
@@ -13,13 +13,15 @@ except NameError:
     # Python 2.3
     from sets import Set as set
 
+import itertools
 from time import time
+
 import Code
 import Errors
 import Parsing
 import Version
 from Scanning import PyrexScanner, FileSourceDescriptor
-from Errors import PyrexError, CompileError, InternalError, error
+from Errors import PyrexError, CompileError, InternalError, AbortError, error, warning
 from Symtab import BuiltinScope, ModuleScope
 from Cython import Utils
 from Cython.Utils import open_new_file, replace_suffix
@@ -38,7 +40,7 @@ def dumptree(t):
 def abort_on_errors(node):
     # Stop the pipeline if there are any errors.
     if Errors.num_errors != 0:
-        raise InternalError, "abort"
+        raise AbortError, "pipeline break"
     return node
 
 class CompilationData(object):
@@ -65,8 +67,9 @@ class Context(object):
     #  modules               {string : ModuleScope}
     #  include_directories   [string]
     #  future_directives     [object]
-    
-    def __init__(self, include_directories, compiler_directives):
+    #  language_level        int     currently 2 or 3 for Python 2/3
+
+    def __init__(self, include_directories, compiler_directives, cpp=False, language_level=2):
         #self.modules = {"__builtin__" : BuiltinScope()}
         import Builtin, CythonScope
         self.modules = {"__builtin__" : Builtin.builtin_scope}
@@ -74,56 +77,62 @@ class Context(object):
         self.include_directories = include_directories
         self.future_directives = set()
         self.compiler_directives = compiler_directives
+        self.cpp = cpp
 
         self.pxds = {} # full name -> node tree
 
-        standard_include_path = os.path.abspath(
-            os.path.join(os.path.dirname(__file__), '..', 'Includes'))
+        standard_include_path = os.path.abspath(os.path.normpath(
+            os.path.join(os.path.dirname(__file__), os.path.pardir, 'Includes')))
         self.include_directories = include_directories + [standard_include_path]
 
+        self.set_language_level(language_level)
+
+        self.gdb_debug_outputwriter = None
+
+    def set_language_level(self, level):
+        self.language_level = level
+        if level >= 3:
+            from Future import print_function, unicode_literals
+            self.future_directives.add(print_function)
+            self.future_directives.add(unicode_literals)
+
     def create_pipeline(self, pxd, py=False):
         from Visitor import PrintTree
         from ParseTreeTransforms import WithTransform, NormalizeTree, PostParse, PxdPostParse
         from ParseTreeTransforms import AnalyseDeclarationsTransform, AnalyseExpressionsTransform
         from ParseTreeTransforms import CreateClosureClasses, MarkClosureVisitor, DecoratorTransform
         from ParseTreeTransforms import InterpretCompilerDirectives, TransformBuiltinMethods
-        from TypeInference import MarkAssignments
+        from ParseTreeTransforms import ExpandInplaceOperators
+        from TypeInference import MarkAssignments, MarkOverflowingArithmetic
         from ParseTreeTransforms import AlignFunctionDefinitions, GilCheck
         from AnalysedTreeTransforms import AutoTestDictTransform
         from AutoDocTransforms import EmbedSignature
         from Optimize import FlattenInListTransform, SwitchTransform, IterationTransform
-        from Optimize import OptimizeBuiltinCalls, ConstantFolding, FinalOptimizePhase
+        from Optimize import EarlyReplaceBuiltinCalls, OptimizeBuiltinCalls
+        from Optimize import ConstantFolding, FinalOptimizePhase
         from Optimize import DropRefcountingTransform
         from Buffer import IntroduceBufferAuxiliaryVars
         from ModuleNode import check_c_declarations, check_c_declarations_pxd
 
-        # Temporary hack that can be used to ensure that all result_code's
-        # are generated at code generation time.
-        import Visitor
-        class ClearResultCodes(Visitor.CythonTransform):
-            def visit_ExprNode(self, node):
-                self.visitchildren(node)
-                node.result_code = "<cleared>"
-                return node
-
         if pxd:
             _check_c_declarations = check_c_declarations_pxd
             _specific_post_parse = PxdPostParse(self)
         else:
             _check_c_declarations = check_c_declarations
             _specific_post_parse = None
-            
+
         if py and not pxd:
             _align_function_definitions = AlignFunctionDefinitions(self)
         else:
             _align_function_definitions = None
- 
+
         return [
             NormalizeTree(self),
             PostParse(self),
             _specific_post_parse,
             InterpretCompilerDirectives(self, self.compiler_directives),
             _align_function_definitions,
+            MarkClosureVisitor(self),
             ConstantFolding(),
             FlattenInListTransform(),
             WithTransform(self),
@@ -131,20 +140,21 @@ class Context(object):
             AnalyseDeclarationsTransform(self),
             AutoTestDictTransform(self),
             EmbedSignature(self),
+            EarlyReplaceBuiltinCalls(self),  ## Necessary?
             MarkAssignments(self),
-            TransformBuiltinMethods(self),
+            MarkOverflowingArithmetic(self),
+            TransformBuiltinMethods(self),  ## Necessary?
             IntroduceBufferAuxiliaryVars(self),
             _check_c_declarations,
             AnalyseExpressionsTransform(self),
-            OptimizeBuiltinCalls(self),
+            CreateClosureClasses(self),  ## After all lookups and type inference
+            ExpandInplaceOperators(self),
+            OptimizeBuiltinCalls(self),  ## Necessary?
             IterationTransform(),
             SwitchTransform(),
             DropRefcountingTransform(),
             FinalOptimizePhase(self),
             GilCheck(),
-#            ClearResultCodes(self),
-#            SpecialFunctions(self),
-            #        CreateClosureClasses(context),
             ]
 
     def create_pyx_pipeline(self, options, result, py=False):
@@ -172,13 +182,22 @@ class Context(object):
             from Cython.TestUtils import TreeAssertVisitor
             test_support.append(TreeAssertVisitor())
 
-        return ([
-                create_parse(self),
-            ] + self.create_pipeline(pxd=False, py=py) + test_support + [
-                inject_pxd_code,
-                abort_on_errors,
-                generate_pyx_code,
-            ])
+        if options.gdb_debug:
+            from Cython.Debugger import DebugWriter
+            from ParseTreeTransforms import DebugTransform
+            self.gdb_debug_outputwriter = DebugWriter.CythonDebugWriter(
+                options.output_dir)
+            debug_transform = [DebugTransform(self, options, result)]
+        else:
+            debug_transform = []
+
+        return list(itertools.chain(
+            [create_parse(self)],
+            self.create_pipeline(pxd=False, py=py),
+            test_support,
+            [inject_pxd_code, abort_on_errors],
+            debug_transform,
+            [generate_pyx_code]))
 
     def create_pxd_pipeline(self, scope, module_name):
         def parse_pxd(source_desc):
@@ -195,7 +214,7 @@ class Context(object):
         return [parse_pxd] + self.create_pipeline(pxd=True) + [
             ExtractPxdCode(self),
             ]
-            
+
     def create_py_pipeline(self, options, result):
         return self.create_pyx_pipeline(options, result, py=True)
 
@@ -204,7 +223,7 @@ class Context(object):
         pipeline = self.create_pxd_pipeline(scope, module_name)
         result = self.run_pipeline(pipeline, source_desc)
         return result
-    
+
     def nonfatal_error(self, exc):
         return Errors.report_error(exc)
 
@@ -212,23 +231,29 @@ class Context(object):
         error = None
         data = source
         try:
-            for phase in pipeline:
-                if phase is not None:
-                    if DebugFlags.debug_verbose_pipeline:
-                        print "Entering pipeline phase %r" % phase
-                    data = phase(data)
-        except CompileError, err:
-            # err is set
-            Errors.report_error(err)
-            error = err
+            try:
+                for phase in pipeline:
+                    if phase is not None:
+                        if DebugFlags.debug_verbose_pipeline:
+                            t = time()
+                            print "Entering pipeline phase %r" % phase
+                        data = phase(data)
+                        if DebugFlags.debug_verbose_pipeline:
+                            print "    %.3f seconds" % (time() - t)
+            except CompileError, err:
+                # err is set
+                Errors.report_error(err)
+                error = err
         except InternalError, err:
             # Only raise if there was not an earlier error
             if Errors.num_errors == 0:
                 raise
             error = err
+        except AbortError, err:
+            error = err
         return (error, data)
 
-    def find_module(self, module_name, 
+    def find_module(self, module_name,
             relative_to = None, pos = None, need_pxd = 1):
         # Finds and returns the module scope corresponding to
         # the given relative or absolute module name. If this
@@ -295,7 +320,7 @@ class Context(object):
                 except CompileError:
                     pass
         return scope
-    
+
     def find_pxd_file(self, qualified_name, pos):
         # Search include path for the .pxd file corresponding to the
         # given fully-qualified module name.
@@ -304,13 +329,33 @@ class Context(object):
         # the directory containing the source file is searched first
         # for a dotted filename, and its containing package root
         # directory is searched first for a non-dotted filename.
-        return self.search_include_directories(qualified_name, ".pxd", pos)
+        pxd = self.search_include_directories(qualified_name, ".pxd", pos)
+        if pxd is None: # XXX Keep this until Includes/Deprecated is removed
+            if (qualified_name.startswith('python') or
+                qualified_name in ('stdlib', 'stdio', 'stl')):
+                standard_include_path = os.path.abspath(os.path.normpath(
+                        os.path.join(os.path.dirname(__file__), os.path.pardir, 'Includes')))
+                deprecated_include_path = os.path.join(standard_include_path, 'Deprecated')
+                self.include_directories.append(deprecated_include_path)
+                try:
+                    pxd = self.search_include_directories(qualified_name, ".pxd", pos)
+                finally:
+                    self.include_directories.pop()
+                if pxd:
+                    name = qualified_name
+                    if name.startswith('python'):
+                        warning(pos, "'%s' is deprecated, use 'cpython'" % name, 1)
+                    elif name in ('stdlib', 'stdio'):
+                        warning(pos, "'%s' is deprecated, use 'libc.%s'" % (name, name), 1)
+                    elif name in ('stl'):
+                        warning(pos, "'%s' is deprecated, use 'libcpp.*.*'" % name, 1)
+        return pxd
 
     def find_pyx_file(self, qualified_name, pos):
         # Search include path for the .pyx file corresponding to the
         # given fully-qualified module name, as for find_pxd_file().
         return self.search_include_directories(qualified_name, ".pyx", pos)
-    
+
     def find_include_file(self, filename, pos):
         # Search list of include directories for filename.
         # Reports an error and returns None if not found.
@@ -319,7 +364,7 @@ class Context(object):
         if not path:
             error(pos, "'%s' not found" % filename)
         return path
-    
+
     def search_include_directories(self, qualified_name, suffix, pos,
                                    include=False):
         # Search the list of include directories for the given
@@ -349,17 +394,17 @@ class Context(object):
 
         for dir in dirs:
             path = os.path.join(dir, dotted_filename)
-            if os.path.exists(path):
+            if Utils.path_exists(path):
                 return path
             if not include:
                 package_dir = self.check_package_dir(dir, package_names)
                 if package_dir is not None:
                     path = os.path.join(package_dir, module_filename)
-                    if os.path.exists(path):
+                    if Utils.path_exists(path):
                         return path
                     path = os.path.join(dir, package_dir, module_name,
                                         package_filename)
-                    if os.path.exists(path):
+                    if Utils.path_exists(path):
                         return path
         return None
 
@@ -372,20 +417,12 @@ class Context(object):
             dir = parent
         return dir
 
-    def is_package_dir(self, dir):
-        package_init = os.path.join(dir, "__init__.py")
-        return os.path.exists(package_init) or \
-            os.path.exists(package_init + "x") # same with .pyx
-
     def check_package_dir(self, dir, package_names):
-        package_dir = os.path.join(dir, *package_names)
-        if not os.path.exists(package_dir):
-            return None
         for dirname in package_names:
             dir = os.path.join(dir, dirname)
             if not self.is_package_dir(dir):
                 return None
-        return package_dir
+        return dir
 
     def c_file_out_of_date(self, source_path):
         c_path = Utils.replace_suffix(source_path, ".c")
@@ -408,16 +445,18 @@ class Context(object):
             if dep_path and Utils.file_newer_than(dep_path, c_time):
                 return 1
         return 0
-    
+
     def find_cimported_module_names(self, source_path):
         return [ name for kind, name in self.read_dependency_file(source_path)
                  if kind == "cimport" ]
 
     def is_package_dir(self, dir_path):
         #  Return true if the given directory is a package directory.
-        for filename in ("__init__.py", "__init__.pyx"):
+        for filename in ("__init__.py",
+                         "__init__.pyx",
+                         "__init__.pxd"):
             path = os.path.join(dir_path, filename)
-            if os.path.exists(path):
+            if Utils.path_exists(path):
                 return 1
 
     def read_dependency_file(self, source_path):
@@ -440,7 +479,7 @@ class Context(object):
         # Find a top-level module, creating a new one if needed.
         scope = self.lookup_submodule(name)
         if not scope:
-            scope = ModuleScope(name, 
+            scope = ModuleScope(name,
                 parent_module = None, context = self)
             self.modules[name] = scope
         return scope
@@ -448,7 +487,8 @@ class Context(object):
     def parse(self, source_desc, scope, pxd, full_module_name):
         if not isinstance(source_desc, FileSourceDescriptor):
             raise RuntimeError("Only file sources for code supported")
-        source_filename = Utils.encode_filename(source_desc.filename)
+        source_filename = source_desc.filename
+        scope.cpp = self.cpp
         # Parse the given source file and return a parse tree.
         try:
             f = Utils.open_source_file(source_filename, "rU")
@@ -485,13 +525,15 @@ class Context(object):
         names.reverse()
         return ".".join(names)
 
-    def setup_errors(self, options):
+    def setup_errors(self, options, result):
+        Errors.reset() # clear any remaining error state
         if options.use_listing_file:
             result.listing_file = Utils.replace_suffix(source, ".lis")
-            Errors.open_listing_file(result.listing_file,
-                echo_to_stderr = options.errors_to_stderr)
+            path = result.listing_file
         else:
-            Errors.open_listing_file(None)
+            path = None
+        Errors.open_listing_file(path=path,
+                                 echo_to_stderr=options.errors_to_stderr)
 
     def teardown_errors(self, err, options, result):
         source_desc = result.compilation_source.source_desc
@@ -507,15 +549,6 @@ class Context(object):
             except EnvironmentError:
                 pass
             result.c_file = None
-        if result.c_file and not options.c_only and c_compile:
-            result.object_file = c_compile(result.c_file,
-                verbose_flag = options.show_version,
-                cplus = options.cplus)
-            if not options.obj_only and c_link:
-                result.extension_file = c_link(result.object_file,
-                    extra_objects = options.objects,
-                    verbose_flag = options.show_version,
-                    cplus = options.cplus)
 
 def create_parse(context):
     def parse(compsrc):
@@ -547,7 +580,7 @@ def create_default_resultobj(compilation_source, options):
 
 def run_pipeline(source, options, full_module_name = None):
     # Set up context
-    context = Context(options.include_path, options.compiler_directives)
+    context = options.create_context()
 
     # Set up source object
     cwd = os.getcwd()
@@ -557,18 +590,19 @@ def run_pipeline(source, options, full_module_name = None):
 
     # Set up result object
     result = create_default_resultobj(source, options)
-    
+
     # Get pipeline
     if source_desc.filename.endswith(".py"):
         pipeline = context.create_py_pipeline(options, result)
     else:
         pipeline = context.create_pyx_pipeline(options, result)
 
-    context.setup_errors(options)
+    context.setup_errors(options, result)
     err, enddata = context.run_pipeline(pipeline, source)
     context.teardown_errors(err, options, result)
     return result
 
+
 #------------------------------------------------------------------------
 #
 #  Main Python entry points
@@ -588,7 +622,7 @@ class CompilationSource(object):
 class CompilationOptions(object):
     """
     Options to the Cython compiler:
-    
+
     show_version      boolean   Display version number
     use_listing_file  boolean   Generate a .lis file
     errors_to_stderr  boolean   Echo errors to stderr when using .lis
@@ -602,18 +636,13 @@ class CompilationOptions(object):
     quiet             boolean   Don't print source names in recursive mode
     compiler_directives  dict      Overrides for pragma options (see Options.py)
     evaluate_tree_assertions boolean  Test support: evaluate parse tree assertions
-    
-    Following options are experimental and only used on MacOSX:
-    
-    c_only            boolean   Stop after generating C file (default)
-    obj_only          boolean   Stop after compiling to .o file
-    objects           [string]  Extra .o files to link with
+    language_level    integer   The Python language level: 2 or 3
+
     cplus             boolean   Compile as c++ code
     """
-    
-    def __init__(self, defaults = None, c_compile = 0, c_link = 0, **kw):
+
+    def __init__(self, defaults = None, **kw):
         self.include_path = []
-        self.objects = []
         if defaults:
             if isinstance(defaults, CompilationOptions):
                 defaults = defaults.__dict__
@@ -621,16 +650,16 @@ class CompilationOptions(object):
             defaults = default_options
         self.__dict__.update(defaults)
         self.__dict__.update(kw)
-        if c_compile:
-            self.c_only = 0
-        if c_link:
-            self.obj_only = 0
+
+    def create_context(self):
+        return Context(self.include_path, self.compiler_directives,
+                      self.cplus, self.language_level)
 
 
 class CompilationResult(object):
     """
     Results from the Cython compiler:
-    
+
     c_file           string or None   The generated C source file
     h_file           string or None   The generated C header file
     i_file           string or None   The generated .pxi file
@@ -641,7 +670,7 @@ class CompilationResult(object):
     num_errors       integer          Number of compilation errors
     compilation_source CompilationSource
     """
-    
+
     def __init__(self):
         self.c_file = None
         self.h_file = None
@@ -658,10 +687,10 @@ class CompilationResultSet(dict):
     Results from compiling multiple Pyrex source files. A mapping
     from source file paths to CompilationResult instances. Also
     has the following attributes:
-    
+
     num_errors   integer   Total number of compilation errors
     """
-    
+
     num_errors = 0
 
     def add(self, source, result):
@@ -672,7 +701,7 @@ class CompilationResultSet(dict):
 def compile_single(source, options, full_module_name = None):
     """
     compile_single(source, options, full_module_name)
-    
+
     Compile the given Pyrex implementation file and return a CompilationResult.
     Always compiles a single file; does not perform timestamp checking or
     recursion.
@@ -683,11 +712,12 @@ def compile_single(source, options, full_module_name = None):
 def compile_multiple(sources, options):
     """
     compile_multiple(sources, options)
-    
+
     Compiles the given sequence of Pyrex implementation files and returns
     a CompilationResultSet. Performs timestamp checking and/or recursion
     if these are specified in the options.
     """
+    context = options.create_context()
     sources = [os.path.abspath(source) for source in sources]
     processed = set()
     results = CompilationResultSet()
@@ -717,19 +747,17 @@ def compile_multiple(sources, options):
                             "Cannot find .pyx file for cimported module '%s'\n" % module_name)
     return results
 
-def compile(source, options = None, c_compile = 0, c_link = 0,
-            full_module_name = None, **kwds):
+def compile(source, options = None, full_module_name = None, **kwds):
     """
     compile(source [, options], [, <option> = <value>]...)
-    
+
     Compile one or more Pyrex implementation files, with optional timestamp
     checking and recursing on dependecies. The source argument may be a string
     or a sequence of strings If it is a string and no recursion or timestamp
     checking is requested, a CompilationResult is returned, otherwise a
     CompilationResultSet is returned.
     """
-    options = CompilationOptions(defaults = options, c_compile = c_compile,
-        c_link = c_link, **kwds)
+    options = CompilationOptions(defaults = options, **kwds)
     if isinstance(source, basestring) and not options.timestamps \
             and not options.recursive:
         return compile_single(source, options, full_module_name)
@@ -780,8 +808,6 @@ default_options = dict(
     show_version = 0,
     use_listing_file = 0,
     errors_to_stderr = 1,
-    c_only = 1,
-    obj_only = 1,
     cplus = 0,
     output_file = None,
     annotate = False,
@@ -794,14 +820,6 @@ default_options = dict(
     compiler_directives = {},
     evaluate_tree_assertions = False,
     emit_linenums = False,
+    language_level = 2,
+    gdb_debug = False,
 )
-if sys.platform == "mac":
-    from Cython.Mac.MacSystem import c_compile, c_link, CCompilerError
-    default_options['use_listing_file'] = 1
-elif sys.platform == "darwin":
-    from Cython.Mac.DarwinSystem import c_compile, c_link, CCompilerError
-else:
-    c_compile = None
-    c_link = None
-
-
