@@ -1,165 +1,51 @@
+# cython: infer_types=True, language_level=3
 #
-#   Pyrex Scanner
+#   Cython Scanner
 #
 
-#import pickle
-import cPickle as pickle
-
+import sys
 import os
 import platform
-import stat
-import sys
-import codecs
-from time import time
 
 import cython
-cython.declare(EncodedString=object, string_prefixes=object, raw_prefixes=object, IDENT=object)
+cython.declare(EncodedString=object, string_prefixes=object, raw_prefixes=object, IDENT=unicode,
+               print_function=object)
 
 from Cython import Plex, Utils
 from Cython.Plex.Scanners import Scanner
 from Cython.Plex.Errors import UnrecognizedInput
 from Errors import CompileError, error
 from Lexicon import string_prefixes, raw_prefixes, make_lexicon, IDENT
+from Future import print_function
 
 from StringEncoding import EncodedString
 
-try:
-    plex_version = Plex._version
-except AttributeError:
-    plex_version = None
-#print "Plex version:", plex_version ###
-
 debug_scanner = 0
 trace_scanner = 0
 scanner_debug_flags = 0
 scanner_dump_file = None
-binary_lexicon_pickle = 1
-notify_lexicon_unpickling = 0
-notify_lexicon_pickling = 1
 
 lexicon = None
 
-#-----------------------------------------------------------------
-
-def hash_source_file(path):
-    # Try to calculate a hash code for the given source file.
-    # Returns an empty string if the file cannot be accessed.
-    #print "Hashing", path ###
-    try:
-        from hashlib import md5 as new_md5
-    except ImportError:
-        from md5 import new as new_md5
-    f = None
-    try:
-        try:
-            f = open(path, "rU")
-            text = f.read()
-        except IOError, e:
-            print("Unable to hash scanner source file (%s)" % e)
-            return ""
-    finally:
-        if f:
-            f.close()
-    # Normalise spaces/tabs. We don't know what sort of
-    # space-tab substitution the file may have been
-    # through, so we replace all spans of spaces and
-    # tabs by a single space.
-    import re
-    text = re.sub("[ \t]+", " ", text)
-    hash = new_md5(text.encode("ASCII")).hexdigest()
-    return hash
-
-def open_pickled_lexicon(expected_hash):
-    # Try to open pickled lexicon file and verify that
-    # it matches the source file. Returns the opened
-    # file if successful, otherwise None. ???
-    global lexicon_pickle
-    f = None
-    result = None
-    if os.path.exists(lexicon_pickle):
-        try:
-            f = open(lexicon_pickle, "rb")
-            actual_hash = pickle.load(f)
-            if actual_hash == expected_hash:
-                result = f
-                f = None
-            else:
-                print("Lexicon hash mismatch:")       ###
-                print("   expected " + expected_hash) ###
-                print("   got     " + actual_hash)    ###
-        except (IOError, pickle.UnpicklingError), e:
-            print("Warning: Unable to read pickled lexicon " + lexicon_pickle)
-            print(e)
-    if f:
-        f.close()
-    return result
-
-def try_to_unpickle_lexicon():
-    global lexicon, lexicon_pickle, lexicon_hash
-    dir = os.path.dirname(__file__)
-    source_file = os.path.join(dir, "Lexicon.py")
-    lexicon_hash = hash_source_file(source_file)
-    lexicon_pickle = os.path.join(dir, "Lexicon.pickle")
-    f = open_pickled_lexicon(lexicon_hash)
-    if f:
-        if notify_lexicon_unpickling:
-            t0 = time()
-            print("Unpickling lexicon...")
-        try:
-            lexicon = pickle.load(f)
-        except Exception, e:
-            print "WARNING: Exception while loading lexicon pickle, regenerating"
-            print e
-            lexicon = None
-        f.close()
-        if notify_lexicon_unpickling:
-            t1 = time()
-            print("Done (%.2f seconds)" % (t1 - t0))
-
-def create_new_lexicon():
-    global lexicon
-    t0 = time()
-    print("Creating lexicon...")
-    lexicon = make_lexicon()
-    t1 = time()
-    print("Done (%.2f seconds)" % (t1 - t0))
-
-def pickle_lexicon():
-    f = None
-    try:
-        f = open(lexicon_pickle, "wb")
-    except IOError:
-        print("Warning: Unable to save pickled lexicon in " + lexicon_pickle)
-    if f:
-        if notify_lexicon_pickling:
-            t0 = time()
-            print("Pickling lexicon...")
-        pickle.dump(lexicon_hash, f, binary_lexicon_pickle)
-        pickle.dump(lexicon, f, binary_lexicon_pickle)
-        f.close()
-        if notify_lexicon_pickling:
-            t1 = time()
-            print("Done (%.2f seconds)" % (t1 - t0))
-
 def get_lexicon():
     global lexicon
-    if not lexicon and plex_version is None:
-        try_to_unpickle_lexicon()
     if not lexicon:
-        create_new_lexicon()
-        if plex_version is None:
-            pickle_lexicon()
+        lexicon = make_lexicon()
     return lexicon
-    
+
 #------------------------------------------------------------------
 
-reserved_words = [
-    "global", "include", "ctypedef", "cdef", "def", "class",
-    "print", "del", "pass", "break", "continue", "return",
-    "raise", "import", "exec", "try", "except", "finally",
-    "while", "if", "elif", "else", "for", "in", "assert",
-    "and", "or", "not", "is", "in", "lambda", "from", "yield",
-    "cimport", "by", "with", "cpdef", "DEF", "IF", "ELIF", "ELSE"
+py_reserved_words = [
+    "global", "def", "class", "print", "del", "pass", "break",
+    "continue", "return", "raise", "import", "exec", "try",
+    "except", "finally", "while", "if", "elif", "else", "for",
+    "in", "assert", "and", "or", "not", "is", "in", "lambda",
+    "from", "yield", "with", "nonlocal",
+]
+
+pyx_reserved_words = py_reserved_words + [
+    "include", "ctypedef", "cdef", "cpdef",
+    "cimport", "by", "DEF", "IF", "ELIF", "ELSE"
 ]
 
 class Method(object):
@@ -167,38 +53,27 @@ class Method(object):
     def __init__(self, name):
         self.name = name
         self.__name__ = name # for Plex tracing
-    
+
     def __call__(self, stream, text):
         return getattr(stream, self.name)(text)
 
 #------------------------------------------------------------------
 
-def build_resword_dict():
-    d = {}
-    for word in reserved_words:
-        d[word] = 1
-    return d
-
-cython.declare(resword_dict=object)
-resword_dict = build_resword_dict()
-
-#------------------------------------------------------------------
-
 class CompileTimeScope(object):
 
     def __init__(self, outer = None):
         self.entries = {}
         self.outer = outer
-    
+
     def declare(self, name, value):
         self.entries[name] = value
-    
+
     def lookup_here(self, name):
         return self.entries[name]
-        
+
     def __contains__(self, name):
         return name in self.entries
-    
+
     def lookup(self, name):
         try:
             return self.lookup_here(name)
@@ -215,7 +90,10 @@ def initial_compile_time_env():
         'UNAME_VERSION', 'UNAME_MACHINE')
     for name, value in zip(names, platform.uname()):
         benv.declare(name, value)
-    import __builtin__ as builtins
+    try:
+        import __builtin__ as builtins
+    except ImportError:
+        import builtins
     names = ('False', 'True',
         'abs', 'bool', 'chr', 'cmp', 'complex', 'dict', 'divmod', 'enumerate',
         'float', 'hash', 'hex', 'int', 'len', 'list', 'long', 'map', 'max', 'min',
@@ -236,11 +114,23 @@ class SourceDescriptor(object):
     """
     A SourceDescriptor should be considered immutable.
     """
+    _file_type = 'pyx'
+
     _escaped_description = None
     _cmp_name = ''
     def __str__(self):
         assert False # To catch all places where a descriptor is used directly as a filename
-    
+
+    def set_file_type_from_name(self, filename):
+        name, ext = os.path.splitext(filename)
+        self._file_type = ext in ('.pyx', '.pxd', '.py') and ext[1:] or 'pyx'
+
+    def is_cython_file(self):
+        return self._file_type in ('pyx', 'pxd')
+
+    def is_python_file(self):
+        return self._file_type == 'py'
+
     def get_escaped_description(self):
         if self._escaped_description is None:
             self._escaped_description = \
@@ -277,22 +167,24 @@ class FileSourceDescriptor(SourceDescriptor):
     the position()-tuple.
     """
     def __init__(self, filename):
+        filename = Utils.decode_filename(filename)
         self.filename = filename
+        self.set_file_type_from_name(filename)
         self._cmp_name = filename
-    
+
     def get_lines(self, encoding=None, error_handling=None):
-        if not encoding:
-            return Utils.open_source_file(self.filename)
-        else:
-            return codecs.open(self.filename, "rU", encoding=encoding,
-                               errors=error_handling)
-    
+        return Utils.open_source_file(
+            self.filename, encoding=encoding,
+            error_handling=error_handling,
+            # newline normalisation is costly before Py2.6
+            require_normalised_newlines=False)
+
     def get_description(self):
         return self.filename
-    
+
     def get_filenametable_entry(self):
         return self.filename
-    
+
     def __eq__(self, other):
         return isinstance(other, FileSourceDescriptor) and self.filename == other.filename
 
@@ -309,9 +201,10 @@ class StringSourceDescriptor(SourceDescriptor):
     """
     def __init__(self, name, code):
         self.name = name
+        #self.set_file_type_from_name(name)
         self.codelines = [x + "\n" for x in code.split("\n")]
         self._cmp_name = name
-    
+
     def get_lines(self, encoding=None, error_handling=None):
         if not encoding:
             return self.codelines
@@ -343,7 +236,7 @@ class PyrexScanner(Scanner):
     #  compile_time_eval  boolean  In a true conditional compilation context
     #  compile_time_expr  boolean  In a compile-time expression context
 
-    def __init__(self, file, filename, parent_scanner = None, 
+    def __init__(self, file, filename, parent_scanner = None,
                  scope = None, context = None, source_encoding=None, parse_comments=True, initial_pos=None):
         Scanner.__init__(self, get_lexicon(), file, filename, initial_pos)
         if parent_scanner:
@@ -360,6 +253,12 @@ class PyrexScanner(Scanner):
             self.compile_time_expr = 0
         self.parse_comments = parse_comments
         self.source_encoding = source_encoding
+        if filename.is_python_file():
+            self.in_python_file = True
+            self.keywords = cython.set(py_reserved_words)
+        else:
+            self.in_python_file = False
+            self.keywords = cython.set(pyx_reserved_words)
         self.trace = trace_scanner
         self.indentation_stack = [0]
         self.indentation_char = None
@@ -370,8 +269,8 @@ class PyrexScanner(Scanner):
 
     def commentline(self, text):
         if self.parse_comments:
-            self.produce('commentline', text)    
-    
+            self.produce('commentline', text)
+
     def current_level(self):
         return self.indentation_stack[-1]
 
@@ -387,14 +286,14 @@ class PyrexScanner(Scanner):
         if self.bracket_nesting_level == 0:
             self.begin('INDENT')
             self.produce('NEWLINE', '')
-    
+
     string_states = {
         "'":   'SQ_STRING',
         '"':   'DQ_STRING',
         "'''": 'TSQ_STRING',
         '"""': 'TDQ_STRING'
     }
-    
+
     def begin_string_action(self, text):
         if text[:1] in string_prefixes:
             text = text[1:]
@@ -402,11 +301,11 @@ class PyrexScanner(Scanner):
             text = text[1:]
         self.begin(self.string_states[text])
         self.produce('BEGIN_STRING')
-    
+
     def end_string_action(self, text):
         self.begin('')
         self.produce('END_STRING')
-    
+
     def unclosed_string_action(self, text):
         self.end_string_action(text)
         self.error("Unclosed string literal")
@@ -459,8 +358,15 @@ class PyrexScanner(Scanner):
         except UnrecognizedInput:
             self.error("Unrecognized character")
         if sy == IDENT:
-            if systring in resword_dict:
-                sy = systring
+            if systring in self.keywords:
+                if systring == u'print' and print_function in self.context.future_directives:
+                    self.keywords.discard('print')
+                    systring = EncodedString(systring)
+                elif systring == u'exec' and self.context.language_level >= 3:
+                    self.keywords.discard('exec')
+                    systring = EncodedString(systring)
+                else:
+                    sy = systring
             else:
                 systring = EncodedString(systring)
         self.sy = sy
@@ -472,16 +378,24 @@ class PyrexScanner(Scanner):
             else:
                 t = "%s %s" % (self.sy, self.systring)
             print("--- %3d %2d %s" % (line, col, t))
-    
+
+    def peek(self):
+        saved = self.sy, self.systring
+        self.next()
+        next = self.sy, self.systring
+        self.unread(*next)
+        self.sy, self.systring = saved
+        return next
+
     def put_back(self, sy, systring):
         self.unread(self.sy, self.systring)
         self.sy = sy
         self.systring = systring
-    
+
     def unread(self, token, value):
         # This method should be added to Plex
         self.queue.insert(0, (token, value))
-    
+
     def error(self, message, pos = None, fatal = True):
         if pos is None:
             pos = self.position()
@@ -489,25 +403,29 @@ class PyrexScanner(Scanner):
             err = error(pos, "Possible inconsistent indentation")
         err = error(pos, message)
         if fatal: raise err
-        
+
     def expect(self, what, message = None):
         if self.sy == what:
             self.next()
         else:
             self.expected(what, message)
-    
+
     def expect_keyword(self, what, message = None):
         if self.sy == IDENT and self.systring == what:
             self.next()
         else:
             self.expected(what, message)
-    
+
     def expected(self, what, message = None):
         if message:
             self.error(message)
         else:
-            self.error("Expected '%s'" % what)
-        
+            if self.sy == IDENT:
+                found = self.systring
+            else:
+                found = self.sy
+            self.error("Expected '%s', found '%s'" % (what, found))
+
     def expect_indent(self):
         self.expect('INDENT',
             "Expected an increase in indentation level")
