@@ -3,7 +3,7 @@
 #
 
 import re
-from cStringIO import StringIO
+from StringIO import StringIO
 from Scanning import PyrexScanner, StringSourceDescriptor
 from Symtab import BuiltinScope, ModuleScope
 import Symtab
@@ -23,18 +23,18 @@ class StringParseContext(Main.Context):
     def __init__(self, include_directories, name):
         Main.Context.__init__(self, include_directories, {})
         self.module_name = name
-        
+
     def find_module(self, module_name, relative_to = None, pos = None, need_pxd = 1):
         if module_name != self.module_name:
             raise AssertionError("Not yet supporting any cimports/includes from string code snippets")
         return ModuleScope(module_name, parent_module = None, context = self)
-        
+
 def parse_from_strings(name, code, pxds={}, level=None, initial_pos=None):
     """
     Utility method to parse a (unicode) string of code. This is mostly
     used for internal Cython compiler purposes (creating code snippets
     that transforms should emit, as well as unit testing).
-    
+
     code - a unicode string containing Cython (module-level) code
     name - a descriptive name for the code source (to use in error messages etc.)
     """
@@ -54,12 +54,13 @@ def parse_from_strings(name, code, pxds={}, level=None, initial_pos=None):
     context = StringParseContext([], name)
     scope = context.find_module(module_name, pos = initial_pos, need_pxd = 0)
 
-    buf = StringIO(code.encode(encoding))
+    buf = StringIO(code)
 
     scanner = PyrexScanner(buf, code_source, source_encoding = encoding,
                      scope = scope, context = context, initial_pos = initial_pos)
     if level is None:
         tree = Parsing.p_module(scanner, 0, module_name)
+        tree.scope = scope
     else:
         tree = Parsing.p_code(scanner, level=level)
     return tree
@@ -77,7 +78,7 @@ class ApplyPositionAndCopy(TreeCopier):
     def __init__(self, pos):
         super(ApplyPositionAndCopy, self).__init__()
         self.pos = pos
-        
+
     def visit_Node(self, node):
         copy = super(ApplyPositionAndCopy, self).visit_Node(node)
         copy.pos = self.pos
@@ -86,7 +87,7 @@ class ApplyPositionAndCopy(TreeCopier):
 class TemplateTransform(VisitorTransform):
     """
     Makes a copy of a template tree while doing substitutions.
-    
+
     A dictionary "substitutions" should be passed in when calling
     the transform; mapping names to replacement nodes. Then replacement
     happens like this:
@@ -102,30 +103,34 @@ class TemplateTransform(VisitorTransform):
 
     Also a list "temps" should be passed. Any names listed will
     be transformed into anonymous, temporary names.
-   
+
     Currently supported for tempnames is:
     NameNode
     (various function and class definition nodes etc. should be added to this)
-    
+
     Each replacement node gets the position of the substituted node
     recursively applied to every member node.
     """
 
+    temp_name_counter = 0
+
     def __call__(self, node, substitutions, temps, pos):
         self.substitutions = substitutions
         self.pos = pos
         tempmap = {}
         temphandles = []
         for temp in temps:
-            handle = UtilNodes.TempHandle(PyrexTypes.py_object_type)
+            TemplateTransform.temp_name_counter += 1
+            handle = "__tmpvar_%d" % TemplateTransform.temp_name_counter
+#            handle = UtilNodes.TempHandle(PyrexTypes.py_object_type)
             tempmap[temp] = handle
-            temphandles.append(handle)
+#            temphandles.append(handle)
         self.tempmap = tempmap
         result = super(TemplateTransform, self).__call__(node)
-        if temps:
-            result = UtilNodes.TempsBlockNode(self.get_pos(node),
-                                              temps=temphandles,
-                                              body=result)
+#        if temps:
+#            result = UtilNodes.TempsBlockNode(self.get_pos(node),
+#                                              temps=temphandles,
+#                                              body=result)
         return result
 
     def get_pos(self, node):
@@ -143,7 +148,7 @@ class TemplateTransform(VisitorTransform):
                 c.pos = self.pos
             self.visitchildren(c)
             return c
-    
+
     def try_substitution(self, node, key):
         sub = self.substitutions.get(key)
         if sub is not None:
@@ -152,12 +157,13 @@ class TemplateTransform(VisitorTransform):
             return ApplyPositionAndCopy(pos)(sub)
         else:
             return self.visit_Node(node) # make copy as usual
-            
+
     def visit_NameNode(self, node):
         temphandle = self.tempmap.get(node.name)
         if temphandle:
+            return NameNode(pos=node.pos, name=temphandle)
             # Replace name with temporary
-            return temphandle.ref(self.get_pos(node))
+            #return temphandle.ref(self.get_pos(node))
         else:
             return self.try_substitution(node, node.name)
 
@@ -168,7 +174,7 @@ class TemplateTransform(VisitorTransform):
             return self.try_substitution(node, node.expr.name)
         else:
             return self.visit_Node(node)
-    
+
 def copy_code_tree(node):
     return TreeCopier()(node)
 
@@ -180,12 +186,12 @@ def strip_common_indent(lines):
     minindent = min([len(INDENT_RE.match(x).group(0)) for x in lines])
     lines = [x[minindent:] for x in lines]
     return lines
-    
+
 class TreeFragment(object):
     def __init__(self, code, name="(tree fragment)", pxds={}, temps=[], pipeline=[], level=None, initial_pos=None):
         if isinstance(code, unicode):
-            def fmt(x): return u"\n".join(strip_common_indent(x.split(u"\n"))) 
-            
+            def fmt(x): return u"\n".join(strip_common_indent(x.split(u"\n")))
+
             fmt_code = fmt(code)
             fmt_pxds = {}
             for key, value in pxds.iteritems():
@@ -196,6 +202,8 @@ class TreeFragment(object):
             if not isinstance(t, StatListNode):
                 t = StatListNode(pos=mod.pos, stats=[t])
             for transform in pipeline:
+                if transform is None:
+                    continue
                 t = transform(t)
             self.root = t
         elif isinstance(code, Node):
